struct HidDescriptor
{
  unsigned char  bLength;
  unsigned char  bDescriptorType;
  unsigned short bcdHID;
  unsigned char  bCountry;
  unsigned char  bNumDescriptors;
  // WINGMAN uses only one Report descriptor :)
  struct DescriptorList {
	  unsigned char  bReportType;
	  unsigned short wReportLength;
  } descriptorlist[1];

} __attribute__((packed));

/*------------------------------------------------------------------------------*/
/* WINGMAN GAMEPAD DEVICE DESCRIPTOR												*/
/*------------------------------------------------------------------------------*/
struct DeviceDescriptor devdesc = {
	.bLength            = 0x12,  //Size of the Descriptor in Bytes (18 bytes)
	.bDescriptorType    = 0x01,  //Constant - Device Descriptor (0x01)
	.bcdUSB             = 0x0110,//USB Specification Number which device complies too. USB 2.0 is reported as 0x0200, USB 1.1 as 0x0110 and USB 1.0 as 0x0100.
	.bDeviceClass       = 0,     //Class Code (Assigned by USB Org)
	.bDeviceSubClass    = 0,     //Subclass Code (Assigned by USB Org)
	.bDeviceProtocol    = 0,     //Protocol Code (Assigned by USB Org)
	.bMaxPacketSize     = 0x08,  //Maximum Packet Size for Zero Endpoint. Valid Sizes are 8, 16, 32, 64
	.idVendor           = 0x046D,//(Logitech Inc.), however PSP will set it to 0x054C (Sony Inc.)
	.idProduct          = 0xC20C,//Wingman
	.bcdDevice          = 0x0103,//Device Release Number
	.iManufacturer      = 0x01,  //Index of Manufacturer String Descriptor
	.iProduct           = 0x02,  //Index of Product String Descriptor, PSP will set it to 0x00
	.iSerialNumber      = 0,     //Index of Product String Descriptor
	.bNumConfigurations = 0x01   //Number of Possible Configurations
};

/*------------------------------------------------------------------------------*/
/* WINGMAN configuration descriptor											*/
/*------------------------------------------------------------------------------*/
struct ConfigDescriptor confdesc = {
	.bLength             = 0x09,   //Size of Descriptor in Bytes
	.bDescriptorType     = 0x02,   //Constant - Configuration Descriptor (0x02)
	.wTotalLength        = 9+9+9+7,//sizeof(configuration + interface + hid + endpoint) descriptors
	.bNumInterfaces      = 0x01,   //Number of Interfaces
	.bConfigurationValue = 0x01,   //Value to use as an argument to select this configuration
	.iConfiguration      = 0x00,   //Index of String Descriptor describing this configuration
	.bmAttributes        = 0x80,   //(USB 1.0 Bus Powered) PSP will change it to 0xC0 (Self Powered)
	.bMaxPower           = 0x01    //2 mA
};

/*------------------------------------------------------------------------------*/
/* WINGMAN interface descriptor												*/
/*------------------------------------------------------------------------------*/
struct InterfaceDescriptor interdesc = {
	.bLength            = 0x09, //Size of Descriptor in Bytes (9 Bytes)
	.bDescriptorType    = 0x04, //Interface Descriptor (0x04)
	.bInterfaceNumber   = 0x00, //Number of Interface
	.bAlternateSetting  = 0x00, //Value used to select alternative setting
	.bNumEndpoints      = 0x01, //Number of Endpoints used for this interface
	.bInterfaceClass    = 0x03, //Class Code (Assigned by USB Org) (HID)
	.bInterfaceSubClass = 0x00, //Subclass Code (Assigned by USB Org)
	.bInterfaceProtocol = 0x00, //Protocol Code (Assigned by USB Org)
	.iInterface         = 0x00  //Index of String Descriptor Describing this interface
};
/*------------------------------------------------------------------------------*/
/* WINGMAN HID descriptor												*/
/*------------------------------------------------------------------------------*/

struct HidDescriptor hiddesc = {
  .bLength			= 0x09,		// Why 9??? when it's 8 bytes!? :P
  .bDescriptorType	= 0x21, 	// HID descriptor
  .bcdHID			= 0x0110,	// HID Class spec version 1.10
  .bCountry			= 0x00,		// no special localisation
  .bNumDescriptors	= 0x01,		// We have only one further REPORT desc
 
  .descriptorlist[0].bReportType	= 0x22,  //REPORT DESCRIPTOR :)
  .descriptorlist[0].wReportLength	= 0x0038 //And it's length

};
  
/*------------------------------------------------------------------------------*/
/* WINGMAN endpoint descriptors												*/
/*------------------------------------------------------------------------------*/
struct EndpointDescriptor endpdesc =
	{
		.bLength          = 0x07,	// Valid length
		.bDescriptorType  = 0x05,	// EP type
		.bEndpointAddress = 0x81,	// EP1 - IN
		.bmAttributes     = 0x03,	// Interrupt Data EP
		.wMaxPacketSize   = 0x0003,	// bytes: #1 - buttons bitmask | #2 - X Axis | #3 - Y Axis
		.bInterval        = 10		// 10ms (~8ms in Windows)
	};
