#include <pspkernel.h>
#include <pspsdk.h>
#include <pspusb.h>
#include <pspusbbus.h>
#include <string.h>
#include <stdio.h>
#include "Descriptor.h"
#include "usb.h"

#include <pspdebug.h>

#define printf	pspDebugScreenPrintf
/*------------------------------------------------------------------------------*/
/* USB driver define															*/
/*------------------------------------------------------------------------------*/

#define DRIVERNAME "USBGamePadDriver"
//WINGMAN'S PID
#define DRIVERPID  (0xC20C)

/*------------------------------------------------------------------------------*/
/* USB driver structs and so on																*/
/*------------------------------------------------------------------------------*/
// Zero EP and our EP
static struct UsbEndpoint UsbEndpoint[2] = { {0,0,0}, {1,0,0} };
// One interface?
static struct UsbInterface UsbInterface = { 0xFFFFFFFF, 0, 1, };
// For hi-speed and full-speed descriptors 
// probably it's stupid, cus HID devices support max 64 kb\s
static struct UsbData UsbData[2];
// String Descriptors, Manufacturer and Device names
static struct StringDescriptor StringDescriptor[2] = { {0x13, 0x3, {0,'L',0,'o',0,'g',0,'i',0,'t',0,'e',0,'c',0,'h',0}}, {0x11, 0x3, {0,'W',0,'I',0,'N',0,'G',0,'M',0,'A',0,'N',0}} };

/*------------------------------------------------------------------------------*/
/* UsbRequest																	*/
/*------------------------------------------------------------------------------*/
static int UsbRequest( int arg1, int arg2, struct DeviceRequest *req )
{

	printf("func24 a1 %08X, a2 %08X, a3 %p\n", arg1, arg2, req);
	printf("ReqType %d, Request %d, Value %x, Index %x, Length %x\n", 
			req->bmRequestType, req->bRequest, req->wValue, req->wIndex,
			req->wLength);
	return( 0 );
}

/*------------------------------------------------------------------------------*/
/* UsbUnknown																	*/
/*------------------------------------------------------------------------------*/
static int UsbUnknown( int arg1, int arg2, int arg3 )
{
	// JUST CHECK IF THIS FUNCTION IS CALLED

	printf("unknown a1 %08X, a2 %08X, a3 %08X\n", arg1, arg2, arg3);

	return( 0 );
}

/*------------------------------------------------------------------------------*/
/* UsbAttach																	*/
/*------------------------------------------------------------------------------*/
static int UsbAttach( int speed, void *arg2, void *arg3 )
{
	// Currently unused
	printf("Attach funct\n");
	return( 0 );
}

/*------------------------------------------------------------------------------*/
/* UsbDetach																	*/
/*------------------------------------------------------------------------------*/
static int UsbDetach( int arg1, int arg2, int arg3 )
{
	// Currently unused
	printf("Detach funct\n");
	return( 0 );
}

/*------------------------------------------------------------------------------*/
/* UsbStartFunc																	*/
/*------------------------------------------------------------------------------*/
static int UsbStartFunc( int size, void *p )
{
	printf("Start funct\n");
    // Let's prepare our UsbData structs
	memset( UsbData, 0, sizeof(UsbData) );
	// The first one (UsbData[0]) is hi-speed device descriptor //////////////////
	memcpy( UsbData[0].devdesc, &devdesc, sizeof(devdesc) );
	// Let's set up pointers for config struct
	UsbData[0].config.pconfdesc = &UsbData[0].confdesc;
	UsbData[0].config.pinterfaces = &UsbData[0].interfaces;
	UsbData[0].config.pinterdesc = &UsbData[0].interdesc;
	UsbData[0].config.pendp = &UsbData[0].endp[0];
	// Now we copy Configuration Descriptor
	memcpy( UsbData[0].confdesc.desc, &confdesc,  sizeof(confdesc) );
    // pointer
	UsbData[0].confdesc.pinterfaces = &UsbData[0].interfaces;
	// What's the second .pinterdesc[1] for?
	UsbData[0].interfaces.pinterdesc[0] = UsbData[0].interdesc.desc;
	// mb 2 interfaces?
	UsbData[0].interfaces.intcount = 1;
	// Now we copy Interface Descriptor
	// WHERE TO PUT HID DESCRIPTOR? :(
	memcpy( UsbData[0].interdesc.desc, &interdesc, sizeof(interdesc) );
    // the last pointer
	UsbData[0].interdesc.pendp = &UsbData[0].endp[0];
	// Now fill our EP (we use only 1 EP, hope PSP allow it)
	memcpy( UsbData[0].endp[0].desc, &endpdesc, sizeof(endpdesc) );
	
	// The second one (UsbData[1]) is full-speed device descriptor //////////////////
	// Everything is the same
	memcpy( UsbData[1].devdesc, &devdesc, sizeof(devdesc) );
	
	UsbData[1].config.pconfdesc = &UsbData[1].confdesc;
	UsbData[1].config.pinterfaces = &UsbData[1].interfaces;
	UsbData[1].config.pinterdesc = &UsbData[1].interdesc;
	UsbData[1].config.pendp = &UsbData[1].endp[0];
	
	memcpy( UsbData[1].confdesc.desc, &confdesc,  sizeof(confdesc) );
	UsbData[1].confdesc.pinterfaces = &UsbData[1].interfaces;
	
	UsbData[1].interfaces.pinterdesc[0] = UsbData[1].interdesc.desc;

	UsbData[1].interfaces.intcount = 1;
	
	memcpy( UsbData[1].interdesc.desc, &interdesc, sizeof(interdesc) );

	UsbData[1].interdesc.pendp = &UsbData[1].endp[0];
	
	memcpy( UsbData[1].endp[0].desc, &endpdesc, sizeof(endpdesc) );

	return( 0 );
}

/*------------------------------------------------------------------------------*/
/* UsbStopFunc																	*/
/*------------------------------------------------------------------------------*/
static int UsbStopFunc( int size, void *p )
{
	// Nothing to do here
	printf("Stop funct\n");
	return( 0 );
}

/*------------------------------------------------------------------------------*/
/* USB driver structure															*/
/*------------------------------------------------------------------------------*/
struct UsbDriver UsbDriver = {
	DRIVERNAME,					// "USBGamePadDriver"
	2,							// Our EP + Zero EP = 2 :)
	UsbEndpoint,
	&UsbInterface,
	&UsbData[0].devdesc[0],
	&UsbData[0].config,
	&UsbData[1].devdesc[0],
	&UsbData[1].config,
	StringDescriptor,
	UsbRequest,
	UsbUnknown,
	UsbAttach,
	UsbDetach,
	0,
	UsbStartFunc,
	UsbStopFunc,
	NULL
};

/*------------------------------------------------------------------------------*/
/* UsbbdRegister																*/
/*------------------------------------------------------------------------------*/
void UsbbdRegister( void )
{
	sceUsbbdRegister( &UsbDriver );
	printf("Activated\n");
}

/*------------------------------------------------------------------------------*/
/* UsbbdUnregister																*/
/*------------------------------------------------------------------------------*/
void UsbbdUnregister( void )
{
	sceUsbbdUnregister( &UsbDriver );
	printf("Deactivated\n");
}

/*------------------------------------------------------------------------------*/
/* UsbStart																		*/
/*------------------------------------------------------------------------------*/
int UsbStart( void )
{
	int ret;
	// Turn on USB bus :) Am I wrong?
	ret = sceUsbStart( PSP_USBBUS_DRIVERNAME, 0, 0 );
	if ( ret != 0 ){ return( -1 ); }
	// Turn on our driver
	ret = sceUsbStart( DRIVERNAME, 0, 0 );
	if ( ret != 0 ){ return( -1 ); }
	// Activate our driver :)
	ret = sceUsbActivate( DRIVERPID );
	if ( ret != 0 ){ return( -1 ); }
	printf("Started\n");
	return( 0 );
	
}

/*------------------------------------------------------------------------------*/
/* UsbStop																		*/
/*------------------------------------------------------------------------------*/
int UsbStop( void )
{
	int ret;
	// Deactivate our driver
	ret = sceUsbDeactivate( DRIVERPID );
	if ( ret != 0 ){ return( -1 ); }
	// Turn of our driver
	ret = sceUsbStop( DRIVERNAME, 0, 0 );
	if ( ret != 0 ){ return( -1 ); }
	// And turn off USB bus
	ret = sceUsbStop( PSP_USBBUS_DRIVERNAME, 0, 0 );
	if ( ret != 0 ){ return( -1 ); }
	printf("Stopped\n");
	return( 0 );
}
